// Include Standard files
#include <inarm.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"

#define BIT2  0x02

// REGISTERs
CSP_GIC_T * const GIC     = ((CSP_GIC_T *)   GIC_BASE_ADDRESS);
CSP_ST_T * const ST0      = ((CSP_ST_T *)   ST0_BASE_ADDRESS);
CSP_PIO_T * const UPIO    = ((CSP_PIO_T *)   UPIO_BASE_ADDRESS);



// VARIABLEs
unsigned long delay=0;
unsigned char flag=0;


// Interrupt -----------------------------------------------------------
__arm __irq void c_extirq1_interrupt(void) {

  // Toggle flag
  if(flag==0)
    flag=1;
  else
    flag=0;

  // End of interrupt
  GIC->EOICR = 0;

  for(delay=0; delay<10000; delay++);

}


void InitButtonIRQ(void) {

  // Interrupt -----------------------------------------------------------
  // Put LDR PC, [PC, #-0xF20] instruction into IRQ vector in order to use
  // hardware interrupt vectoring.
  *(volatile unsigned int *)0x00000018 = 0xE51FFF20;

  // Interrupt IRQ
  // Disable & CLear All Interrupts
  CSP_GIC_SET_IDCR(GIC, 0xFFFFFFFF);
  CSP_GIC_SET_ICCR(GIC, 0xFFFFFFFF);
  // Disable the IRQ1 interrupt on the interrupt controller
  GIC->IDCR =  IBIT_EXTIRQ1;
  // Save the interrupt handler routine pointer and the interrupt priority
  GIC->SVR[EXTIRQ1] = (unsigned int)c_extirq1_interrupt;
  // Store the Source Mode Register
  GIC->SMR[EXTIRQ1] = (LOW_LEVEL_SENSITIVE | PRIOR_7);
  // Clear the interrupt on the interrupt controller
  GIC->ICCR = IBIT_EXTIRQ1 ;
  // Ebnable IT
  GIC->IECR = IBIT_EXTIRQ1 ;
  // Software interrupt
  GIC->ISCR = IBIT_EXTIRQ1 ;

  __enable_interrupt();

}



void InitLed(void) {

  // Enable clock
  UPIO->ECR = 1;

  // Set high - turn off
  UPIO->SODR   = BIT2;

  // Led as output
  UPIO->OER    = BIT2;
}


// End Periphery --------------------------------------------------------------



//*----------------------------------------------------------------------------
//* Function Name       : main
//*----------------------------------------------------------------------------
int main (void)
{

  // Init led
  InitLed();

  // Init button
  InitButtonIRQ();

  // Buzzer
  InitPwm3();
  Pwm3Disable();

  __enable_interrupt();

  // loop forever
  while(1) {

    if(flag) {
      // Beep buzzer
      Pwm3Enable();
      // turn on
      UPIO->CODR   = BIT2;
    }
    else {
      // Stop buzzer
      Pwm3Disable();
      // turn off
      UPIO->SODR   = BIT2;
    }
  }
}
